@@ -15,47 +15,19 @@ BEGIN {
     # this is an attempt to get around it...
 
     # for the cpan shell, and other parsers
-    $VERSION = 'v0.18';
+    $VERSION = 'v0.19';
 
     # for actual inspection
     ${VERSION} 
-        = qv('0.18'); 
+        = qv('0.19'); 
 };
 
+
 # Ensure we get detailed errors while starting up.
 # This is disabled at the bottom of the module.
 use Carp;
 $SIG{__DIE__} = \&Carp::confess;
 
-# This speeds up using the module tree by pre-calculaing meta-queries.
-# NOTE: it is currently not used because of deployment issues.
-use Storable qw(store_fd fd_retrieve);
-BEGIN {
-    if ( $INC{'UR.pm'} ) {
-        my $ur_dir = substr($INC{'UR.pm'}, 0, length($INC{'UR.pm'})-5);
-        #print "STDERR ur_dir is $ur_dir\n";
-        my $dump;
-        foreach my $dir ( '.', $ur_dir ) {
-            if (-f "$dir/ur_core.$VERSION.stor" and -s _) {
-                #print STDERR "Loading rules dump from $dir/ur_core.stor\n";
-                open($dump, "$dir/ur_core.$VERSION.stor");
-                last;
-            } elsif (-f "$dir/ur_core.stor.gz" and -s _) {
-                #print STDERR "Loading gzipped rules dump from $dir/ur_core.stor.gz\n";
-                open($dump, "gzip -dc $dir/ur_core.stor.gz |");
-                last;
-            }
-        }
-        if ($dump) {
-            $UR::DID_LOAD_FROM_DUMP = 1;
-
-            local $/;
-            my $data = fd_retrieve($dump);
-            ($UR::Object::rule_templates, $UR::Object::rules) = @$data;
-        }
-    }
-}
-
 # Ensure that, if the application changes directory, we do not 
 # change where we load modules while running.
 use Cwd;
@@ -129,13 +101,8 @@ require UR::Object;
 require UR::Object::Type;
 
 require UR::Object::Ghost;
-require UR::Object::Inheritance;
 require UR::Object::Type;
 require UR::Object::Property;
-require UR::Object::Property::ID;
-require UR::Object::Property::Unique;
-require UR::Object::Reference;
-require UR::Object::Reference::Property;
 
 
 require UR::BoolExpr::Util;
@@ -166,17 +133,6 @@ UR::Object::Type->define(
     ]
 );
 
-UR::Object::Type->define(
-    class_name => 'UR::Object::Inheritance',
-    extends => ['UR::Object'],
-    id_properties => [qw/class_name parent_class_name/],
-    properties => [
-        parent_class_name                => { is => 'Text', len => 256, source => 'data dictionary' },
-        class_name                       => { is => 'Text', len => 256, source => 'data dictionary' },
-        inheritance_priority             => { is => 'NUMBER', len => 2 },
-    ],
-);
-
 UR::Object::Type->define(
     class_name => "UR::Object::Index",
     id_by => ['indexed_class_name','indexed_property_string'],
@@ -279,10 +235,8 @@ UR::Object::Type->define(
         # There's also a property_meta_by_name() method defined in the class
         direct_property_metas            => { is => 'UR::Object::Property', reverse_as => 'class_meta', is_many => 1 },
         direct_property_names            => { via => 'direct_property_metas', to => 'property_name', is_many => 1 },
-        #direct_id_property_metas         => { is => 'UR::Object::Property', reverse_as => 'class_meta', where => [ is_id => 1 ], is_many => 1 },
-        #direct_id_property_names         => { via => 'direct_property_metas', to => 'property_name', is_many => 1, where => [ is_id => 1 ] },
-        direct_id_property_metas         => { via => 'direct_id_token_metas', to => 'property_meta', is_many => 1 },
-        direct_id_property_names         => { via => 'direct_id_token_metas', to => 'property_name', is_many => 1 },
+        direct_id_property_metas         => { is => 'UR::Object::Property', reverse_as => 'class_meta', where => [ 'is_id true' => 1, -order_by => 'is_id' ], is_many => 1 },
+        direct_id_property_names         => { via => 'direct_id_property_metas', to => 'property_name', is_many => 1 },
 
         ancestry_property_metas          => { via => 'ancestry_class_metas', to => 'direct_property_metas', is_many => 1 },
         ancestry_property_names          => { via => 'ancestry_class_metas', to => 'direct_property_names', is_many => 1 },
@@ -291,29 +245,17 @@ UR::Object::Type->define(
 
         all_property_metas               => { via => 'all_class_metas', to => 'direct_property_metas', is_many => 1 },
         all_property_names               => { via => 'all_property_metas', to => 'property_name', is_many => 1 },
-        #all_id_property_metas            => { via => 'ancestry_property_metas', to => 'all_property_metas', where => [is_id => 1] },
-        all_id_property_metas            => { via => 'all_id_token_metas', to => 'property_meta', is_many => 1 },
-        all_id_property_names            => { via => 'all_id_token_metas', to => 'property_name', is_many => 1 },
+        all_id_property_metas            => { via => 'all_class_metas', to => 'direct_id_property_metas', is_many => 1 },
+        all_id_property_names            => { via => 'all_id_property_metas', to => 'property_name', is_many => 1 },
 
-        # these should go away when the is_id meta-property is working, since they don't seem that useful
-        direct_id_token_metas            => { is => 'UR::Object::Property::ID', reverse_as => 'class_meta', is_many => 1 },
-        direct_id_token_names            => { via => 'direct_id_token_metas', to => 'property_name', is_many => 1 },
-        ancestry_id_token_metas          => { via => 'ancestry_class_metas', to => 'direct_id_token_metas', is_many => 1 },
-        ancestry_id_token_names          => { via => 'ancestry_id_token_metas', to => 'property_name', is_many => 1 },
-        all_id_token_metas               => { via => 'all_class_metas', to => 'direct_id_token_metas', is_many => 1 },
-
-        # Unique contstraint trackers
-        direct_unique_metas              => { is => 'UR::Object::Property::Unique', reverse_as => 'class_meta', is_many => 1 },
-        direct_unique_property_metas     => { via => 'direct_unique_metas', to => 'property_meta', is_many => 1 },
-        direct_unique_property_names     => { via => 'direct_unique_metas', to => 'property_name', is_many => 1 },
-        ancestry_unique_property_metas   => { via => 'ancestry_class_metas', to => 'direct_unique_property_metas', is_many => 1 },
-        ancestry_unique_property_names   => { via => 'ancestry_class_metas', to => 'direct_unique_property_names', is_many => 1 },
-        all_unique_property_metas        => { via => 'all_class_metas', to => 'direct_unique_property_metas', is_many => 1 },
-        all_unique_property_names        => { via => 'all_class_metas', to => 'direct_unique_property_names', is_many => 1 },
+        direct_id_by_property_metas      => { via => 'direct_property_metas', to => '__self__', where => ['id_by true' => 1], is_many => 1, doc => "Properties with 'id_by' metadata, ie. direct object accessor properties" } ,
+        all_id_by_property_metas         => { via => 'all_class_metas', to => 'direct_id_by_property_metas', is_many => 1},
+        direct_reverse_as_property_metas => { via => 'direct_property_metas', to => '__self__', where => ['reverse_as true' => 1], is_many => 1, doc => "Properties with 'reverse_as' metadata, ie. indirect object accessor properties" },
+        all_reverse_as_property_metas    => { via => 'all_class_metas', to => 'direct_reverse_as_property_metas', is_many => 1},
 
         # Datasource related stuff
         direct_column_names              => { via => 'direct_property_metas', to => 'column_name', is_many => 1, where => [column_name => { operator => 'true' }] },
-        direct_id_column_names           => { via => 'get_direct_id_property_metas', to => 'column_name', is_many => 1, where => [column_name => { operator => 'true'}] },
+        direct_id_column_names           => { via => 'direct_id_property_metas', to => 'column_name', is_many => 1, where => [column_name => { operator => 'true'}] },
         ancestry_column_names            => { via => 'ancestry_class_metas', to => 'direct_column_names', is_many => 1 },
         ancestry_id_column_names         => { via => 'ancestry_class_metas', to => 'direct_id_column_names', is_many => 1 },
         # Are these *columnless* properties actually necessary?  The user could just use direct_property_metas(column_name => undef)
@@ -327,12 +269,6 @@ UR::Object::Type->define(
         all_id_column_names              => { via => 'all_class_metas', to => 'direct_id_column_names', is_many => 1 },
         all_columnless_property_metas    => { via => 'all_class_metas', to => 'direct_columnless_property_metas', is_many => 1 },
         all_columnless_property_names    => { via => 'all_class_metas', to => 'direct_columnless_property_names', is_many => 1 },
-
-        # Reference objects
-        reference_metas                  => { is => 'UR::Object::Reference', reverse_as => 'class_meta', is_many => 1 },
-        reference_property_metas         => { is => 'UR::Object::Reference::Property', via => 'reference_metas', to => 'reference_property_metas', is_many => 1 },
-        all_reference_metas              => { via => 'all_class_metas', to => 'reference_metas', is_many => 1 },
-        
     ],    
     unique_constraints => [
         { properties => [qw/type_name/], sql => 'SUPER_FAKE_O2' },
@@ -356,7 +292,7 @@ UR::Object::Type->define(
         default_value                   => { is_optional => 1 },
         valid_values                    => { is => 'ARRAY', is_optional => 1, },
         doc                             => { is => 'Text', len => 1000, is_optional => 1 },
-        is_id                           => { is => 'Boolean', default_value => 0, doc => 'denotes this is an ID property of the class' },
+        is_id                           => { is => 'Integer', default_value => undef, doc => 'denotes this is an ID property of the class, and ranks them' },
         is_optional                     => { is => 'Boolean' , default_value => 0},
         is_transient                    => { is => 'Boolean' , default_value => 0},
         is_constant                     => { is => 'Boolean' , default_value => 0},  # never changes
@@ -392,12 +328,11 @@ UR::Object::Type->define(
         is_dimension                    => { is => 'Boolean', is_optional => 1},
         is_specified_in_module_header   => { is => 'Boolean', default_value => 0 },
         position_in_module_header       => { is => 'Integer', is_optional => 1, doc => "Line in the class definition source's section this property appears" },
-        #rank                            => { is => 'Integer', is_optional => 1, doc => 'Order in which the properties are discovered while parsing the class definition' },
         singular_name                   => { is => 'Text' },
         plural_name                     => { is => 'Text' },
 
         class_meta                      => { is => 'UR::Object::Type', id_by => 'class_name' },
-        unique_metas                    => { is => 'UR::Object::Property::Unique', reverse_as => 'property_meta', is_many => 1 },
+        r_class_meta                    => { is => 'UR::Object::Type', id_by => 'data_type' },
     ],
     unique_constraints => [
         { properties => [qw/property_name type_name/], sql => 'SUPER_FAKE_O4' },
@@ -405,84 +340,6 @@ UR::Object::Type->define(
 );
 
 
-UR::Object::Type->define(
-    class_name => 'UR::Object::Reference::Property',
-    id_properties => [qw/tha_id rank/],
-    properties => [
-        rank                            => { is => 'NUMBER', len => 2, source => 'data dictionary' },
-        tha_id                          => { is => 'Text', len => 128, source => 'data dictionary' },
-        attribute_name                  => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        r_attribute_name                => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        property_name                   => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        r_property_name                 => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-         
-        reference_meta                  => { is => 'UR::Object::Reference', id_by => 'tha_id' },
-
-        class_meta                      => { is => 'UR::Object::Type', via => 'reference_meta', to => 'class_meta' },
-        class_name                      => { via => 'class_meta', to => 'class_name' },
-	property_meta                   => { is => 'UR::Object::Property', id_by => [ 'class_name', 'property_name' ] },
-
-        r_class_meta                    => { is => 'UR::Object::Type', via => 'reference_meta', to => 'r_class_meta' },
-        r_class_name                    => { via => 'r_class_meta', to => 'class_name' },
-        r_property_meta                 => { is => 'UR::Object::Property', id_by => [ 'r_class_name', 'r_property_name'] },
-    ],
-);
-
-UR::Object::Type->define(
-    class_name => 'UR::Object::Reference',
-    id_properties => ['tha_id'],
-    properties => [
-        tha_id                          => { is => 'Text', len => 128, source => 'data dictionary' },
-        class_name                      => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        type_name                       => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        delegation_name                 => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        r_class_name                    => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        r_type_name                     => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        #r_delegation_name               => { is => 'Text', len => 256, is_optional => 0, source => 'data dictionary' },
-        constraint_name                 => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        source                          => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        description                     => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        accessor_name_for_id            => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        accessor_name_for_object        => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-
-        reference_property_metas        => { is => 'UR::Object::Reference::Property', reverse_as => 'reference_meta', is_many => 1 },
-        class_meta                      => { is => 'UR::Object::Type', id_by => 'class_name' },
-        r_class_meta                    => { is => 'UR::Object::Type', id_by => 'r_class_name' },
-        property_meta                   => { is => 'UR::Object::Property', id_by => ['class_name', 'delegation_name'] },
-    ],
-);
-
-UR::Object::Type->define(
-    class_name => 'UR::Object::Property::Unique',
-    id_properties => [qw/type_name unique_group attribute_name/],
-    properties => [
-        class_name                       => { is => 'Text', len => 256, source => 'data dictionary' },
-        type_name                        => { is => 'Text', len => 256, source => 'data dictionary' },
-        property_name                    => { is => 'Text', len => 256, source => 'data dictionary' },
-        attribute_name                   => { is => 'Text', len => 256, source => 'data dictionary' },
-        unique_group                     => { is => 'Text', len => 256, source => 'data dictionary' },
-
-        class_meta                       => { is => 'UR::Object::Type', id_by => 'class_name' },
-        property_meta                    => { is => 'UR::Object::Property', id_by => ['class_name', 'property_name'] },
-    ],
-);
-
-
-UR::Object::Type->define(
-    class_name => 'UR::Object::Property::ID',
-    id_properties => [qw/type_name position/],
-    properties => [
-        position                         => { is => 'NUMBER', len => 2, source => 'data dictionary' },
-        class_name                       => { is => 'Text', len => 256, source => 'data dictionary' },
-        type_name                        => { is => 'Text', len => 256, source => 'data dictionary' },
-        attribute_name                   => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },
-        property_name                    => { is => 'Text', len => 256, source => 'data dictionary' },
-
-        class_meta                       => { is => 'UR::Object::Type', id_by => 'class_name' },
-        property_meta                    => { is => 'UR::Object::Property', id_by => ['class_name', 'property_name'] },
-    ],
-);
-
 UR::Object::Type->define(
     class_name => 'UR::Object::Property::Calculated::From',
     id_properties => [qw/class_name calculated_property_name source_property_name/],
@@ -547,7 +404,7 @@ UR - rich declarative transactional objects
 
 =head1 VERSION
 
-This document describes UR version v0.17.
+This document describes UR version v0.19.
 
 =head1 SYNOPSIS
 
@@ -600,7 +457,7 @@ This document describes UR version v0.17.
     
     $c->fur('scruffy');
 
-    commit();
+    UR::Context->commit();
 
   
 =head1 DESCRIPTION
@@ -803,8 +660,7 @@ See L<UR::Env> for details.
 Class::Autouse
 Cwd
 Data::Dumper
-Date::Calc
-Date::Parse
+Date::Format
 DBI
 File::Basename
 FindBin
