@@ -1,24 +1,10 @@
-package UR::All;
-
-use strict;
-use warnings;
-
-our $VERSION = "0.44_01"; # UR $VERSION;
-
-BEGIN { require above; };
-use UR;
-
+# this is not used by UR, but is present for testing
+# using all of the modules when only some are needed by most apps costs compile time
+require above;
+use Class::Autouse_1_99_02;
 use Command;
 use Command::DynamicSubCommands;
-use Command::Test;
-use Command::Test::Echo;
-use Command::Test::Tree1;
-use Command::Test::Tree1::Echo1;
-use Command::Test::Tree1::Echo2;
-use Command::Tree;
-use Command::V1;
-use Command::V2;
-use Devel::callcount;
+use UR;
 use UR::BoolExpr;
 use UR::BoolExpr::Template;
 use UR::BoolExpr::Template::And;
@@ -35,7 +21,7 @@ use UR::BoolExpr::Template::PropertyComparison::LessOrEqual;
 use UR::BoolExpr::Template::PropertyComparison::LessThan;
 use UR::BoolExpr::Template::PropertyComparison::Like;
 use UR::BoolExpr::Template::PropertyComparison::Matches;
-use UR::BoolExpr::Template::PropertyComparison::NotEquals;
+use UR::BoolExpr::Template::PropertyComparison::NotEqual;
 use UR::BoolExpr::Template::PropertyComparison::NotIn;
 use UR::BoolExpr::Template::PropertyComparison::NotLike;
 use UR::BoolExpr::Template::PropertyComparison::True;
@@ -43,29 +29,21 @@ use UR::BoolExpr::Util;
 use UR::Change;
 use UR::Context;
 use UR::Context::DefaultRoot;
-use UR::Context::ObjectFabricator;
 use UR::Context::Process;
 use UR::Context::Root;
 use UR::Context::Transaction;
+use UR::DBI;
+use UR::DBIx::ContextualFetch;
 use UR::DataSource;
-use UR::DataSource::Code;
 use UR::DataSource::CSV;
+use UR::DataSource::Code;
 use UR::DataSource::Default;
 use UR::DataSource::File;
 use UR::DataSource::FileMux;
 use UR::DataSource::Meta;
-BEGIN {
-    eval { require DBD::mysql };
-    require UR::DataSource::MySQL unless $@;
-}
-BEGIN {
-    eval { require DBD::Oracle };
-    require UR::DataSource::Oracle unless $@;
-}
-BEGIN {
-    eval { require DBD::Pg };
-    require UR::DataSource::Pg unless $@;
-}
+use UR::DataSource::MySQL;
+use UR::DataSource::Oracle;
+use UR::DataSource::Pg;
 use UR::DataSource::RDBMS;
 use UR::DataSource::RDBMS::BitmapIndex;
 use UR::DataSource::RDBMS::Entity;
@@ -77,9 +55,9 @@ use UR::DataSource::RDBMS::Table::View::Default::Text;
 use UR::DataSource::RDBMS::TableColumn;
 use UR::DataSource::RDBMS::TableColumn::View::Default::Text;
 use UR::DataSource::RDBMS::UniqueConstraintColumn;
+use UR::DataSource::RemoteCache;
 use UR::DataSource::SQLite;
 use UR::DataSource::ValueDomain;
-use UR::DBI;
 use UR::Debug;
 use UR::DeletedRef;
 use UR::Env::UR_COMMAND_DUMP_STATUS_MESSAGES;
@@ -100,22 +78,22 @@ use UR::Env::UR_DBI_NO_COMMIT;
 use UR::Env::UR_DEBUG_OBJECT_PRUNING;
 use UR::Env::UR_DEBUG_OBJECT_RELEASE;
 use UR::Env::UR_IGNORE;
+use UR::Env::UR_MOOSE;
 use UR::Env::UR_NR_CPU;
 use UR::Env::UR_STACK_DUMP_ON_DIE;
 use UR::Env::UR_STACK_DUMP_ON_WARN;
+use UR::Env::UR_TEST_FILLDB;
 use UR::Env::UR_TEST_QUIET;
-use UR::Env::UR_USE_ANY;
-use UR::Env::UR_USE_DUMMY_AUTOGENERATED_IDS;
 use UR::Env::UR_USED_LIBS;
 use UR::Env::UR_USED_MODS;
+use UR::Env::UR_USE_DUMMY_AUTOGENERATED_IDS;
 use UR::Exit;
 use UR::ModuleBase;
-use UR::ModuleBuild;
 use UR::ModuleConfig;
 use UR::ModuleLoader;
+use UR::Moose;
 use UR::Namespace;
 use UR::Namespace::Command;
-use UR::Namespace::Command::Base;
 use UR::Namespace::Command::Define;
 use UR::Namespace::Command::Define::Class;
 use UR::Namespace::Command::Define::Datasource;
@@ -128,8 +106,7 @@ use UR::Namespace::Command::Define::Datasource::RdbmsWithAuth;
 use UR::Namespace::Command::Define::Datasource::Sqlite;
 use UR::Namespace::Command::Define::Db;
 use UR::Namespace::Command::Define::Namespace;
-use UR::Namespace::Command::Show::Properties;
-use UR::Namespace::Command::Show::Schema;
+use UR::Namespace::Command::Describe;
 use UR::Namespace::Command::Init;
 use UR::Namespace::Command::List;
 use UR::Namespace::Command::List::Classes;
@@ -156,16 +133,20 @@ use UR::Namespace::Command::Test::Window;
 use UR::Namespace::Command::Update;
 use UR::Namespace::Command::Update::ClassDiagram;
 use UR::Namespace::Command::Update::ClassesFromDb;
-use UR::Namespace::Command::Update::Pod;
 use UR::Namespace::Command::Update::RenameClass;
 use UR::Namespace::Command::Update::RewriteClassHeader;
 use UR::Namespace::Command::Update::SchemaDiagram;
 use UR::Namespace::Command::Update::TabCompletionSpec;
+use UR::Namespace::View::SchemaBrowser::CgiApp;
+use UR::Namespace::View::SchemaBrowser::CgiApp::Base;
+use UR::Namespace::View::SchemaBrowser::CgiApp::Class;
+use UR::Namespace::View::SchemaBrowser::CgiApp::File;
+use UR::Namespace::View::SchemaBrowser::CgiApp::Index;
+use UR::Namespace::View::SchemaBrowser::CgiApp::Schema;
 use UR::Object;
 use UR::Object::Accessorized;
 use UR::Object::Command::FetchAndDo;
 use UR::Object::Command::List;
-use UR::Object::Command::List::Style;
 use UR::Object::Ghost;
 use UR::Object::Index;
 use UR::Object::Iterator;
@@ -184,6 +165,13 @@ use UR::Object::Type::Initializer;
 use UR::Object::Type::InternalAPI;
 use UR::Object::Type::ModuleWriter;
 use UR::Object::Type::View::Default::Text;
+use UR::Object::Type::View::Default::Umlet;
+use UR::Object::Umlet;
+use UR::Object::Umlet::Class;
+use UR::Object::Umlet::Diagram;
+use UR::Object::Umlet::Other;
+use UR::Object::Umlet::PictureElement;
+use UR::Object::Umlet::Relation;
 use UR::Object::Value;
 use UR::Object::View;
 use UR::Object::View::Aspect;
@@ -194,16 +182,21 @@ use UR::Object::View::Default::Text;
 use UR::Object::View::Lister::Text;
 use UR::Object::View::Toolkit;
 use UR::Object::View::Toolkit::Text;
+use UR::Object::View::Toolkit::Umlet;
 use UR::ObjectDeprecated;
 use UR::ObjectV001removed;
 use UR::ObjectV04removed;
 use UR::Observer;
-use UR::DBI::Report;
+use UR::Report;
+use UR::Service::DataSourceProxy;
+use UR::Service::JsonRpcServer;
 use UR::Service::RPC::Executer;
 use UR::Service::RPC::Message;
 use UR::Service::RPC::Server;
 use UR::Service::RPC::TcpConnectionListener;
 use UR::Singleton;
+use UR::Test;
+use UR::Time;
 use UR::Util;
 use UR::Value;
 use UR::Value::ARRAY;
@@ -212,9 +205,8 @@ use UR::Value::CSV;
 use UR::Value::DateTime;
 use UR::Value::Decimal;
 use UR::Value::DirectoryPath;
-use UR::Value::FilePath;
-use UR::Value::FilesystemPath;
 use UR::Value::FOF;
+use UR::Value::FilePath;
 use UR::Value::HASH;
 use UR::Value::Integer;
 use UR::Value::Iterator;
@@ -225,49 +217,3 @@ use UR::Value::Set;
 use UR::Value::Text;
 use UR::Value::URL;
 use UR::Vocabulary;
-
-# optional elements
-if (eval "use Net::HTTPServer") {
-    my $rv = eval "UR::Namespace::View::SchemaBrowser::CgiApp;"
-             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Base;"
-             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Class;"
-             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::File;"
-             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Index;"
-             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Schema;"
-             && eval "use UR::Service::JsonRpcServer;";
-    die $@ unless ($rv);
-}
-
-
-if (eval "use Xml::LibXSLT") {
-    my $rv = eval "use UR::Object::View::Default::Html;"
-             && eval "use UR::Object::View::Default::Xsl;"
-             && eval "use UR::Object::Set::View::Default::Xml;"
-             && eval "use UR::Object::View::Default::Xml;"
-             && eval "use UR::Object::Type::View::Default::Xml;"
-             ;
-    die $@ unless ($rv);
-}
- 
-
-1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-UR::All
-
-=head1 SYNOPSIS
-
- use UR::All;
-
-=head1 DESCRIPTION
-
-This module exists to let software preload everything in the distribution
-
-It is slower than "use UR", but is good for things like FastCGI servers.
-
-=cut
