@@ -3,12 +3,11 @@ package UR::Namespace::Command::Define::Db;
 use warnings;
 use strict;
 use UR;
-our $VERSION = "0.46"; # UR $VERSION;
 use IO::File; # required to import symbols used below
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => "UR::Namespace::Command::Base",
+    is => "UR::Namespace::Command",
     has_input => [
         uri => {
             is => 'Text',
@@ -77,6 +76,8 @@ sub data_source_module_pathname {
 sub execute {
     my $self = shift;
 
+    $self->_init or return;
+
     my $namespace = $self->namespace_name;
     unless ($namespace) {
         $self->error_message("This command must be run from a namespace directory.");
@@ -151,8 +152,8 @@ sub execute {
     my $method = '_post_module_written_' . lc($driver);
     $self->$method($module_path,$server);
 
-    unless (UR::Object::Type->use_module_with_namespace_constraints($class_name)) {
-    #if ($@) {
+    eval "use $class_name";
+    if ($@) {
         $self->error_message("Error in module $class_name!?: $@");
         return;
     }
@@ -210,33 +211,6 @@ sub _post_module_written_sqlite {
     return 1;
 }
 
-sub _post_module_written_pg {
-    my ($self, $pathname, $server) = @_;
-    return 1;
-}
-
-
-sub _post_module_written_oracle {
-    my ($self, $pathname, $server) = @_;
-    return 1;
-}
-
-sub _post_module_written_mysql {
-    my ($self, $pathname, $server) = @_;
-    return 1;
-}
-
-sub _post_module_written_file {
-    my ($self, $pathname, $server) = @_;
-    return 1;
-}
-
-sub _post_module_written_filemux {
-    my ($self, $pathname, $server) = @_;
-    return 1;
-}
-
-
 
 sub _try_connect {
     my $self = shift;
