@@ -3,11 +3,10 @@ package UR::Namespace::Command::Old::Info;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => 'UR::Namespace::Command::Base',
+    is => 'UR::Namespace::Command',
     has => [
         subject => {
             is_optional => 1,
@@ -28,6 +27,8 @@ sub is_sub_command_delegator { 0;}
 sub execute {
 my($self, $params) = @_;
 
+    $self->_init or return;
+
     my $namespace = $self->namespace_name;
     # FIXME why dosen't require work here?
     eval "use  $namespace";
