@@ -10,11 +10,8 @@ use UR;
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => "UR::Namespace::Command",
+    doc => 'tools for testing and debugging',
 );
 
-sub help_brief {
-   "Perform various tests."
-}
-
 1;
 
