@@ -4,7 +4,6 @@ package UR::Namespace::Command::Update::ClassesFromDb;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 use Text::Diff;
 
 UR::Object::Type->define(
@@ -84,10 +83,26 @@ sub execute {
     # Command parameter checking
     #
     
+#$DB::single=1;
     my $force_check_all_tables = $self->force_check_all_tables;
     my $force_rewrite_all_classes = $self->force_rewrite_all_classes;
     
+    # Hack because some parts of the schema are only visible to the rw user.
+    #my $access_level_param = UR::Command::Param->get(command_id => 'main', name => 'access');
+    #if ($access_level_param) {
+    #    my $access_level = $access_level_param->value;
+    #    unless (defined $access_level and $access_level eq "rw") {
+    #        $access_level_param->value("rw");
+    #    }
+    #}
+
+    $self->_init;
+
     my $namespace = $self->namespace_name;
+    unless ($namespace) {
+        $self->error_message("This command must be run from a namespace directory.");
+        return;
+    }
     $self->status_message("Updating namespace: $namespace\n");
 
     my @namespace_data_sources = $namespace->get_data_sources;
@@ -104,8 +119,7 @@ sub execute {
         if ($specified_class_name_arrayref) {
             $ds_table_list = [
                 map { [$_->data_source, $_->table_name] }
-                grep { $_->data_source }
-                map { $_->__meta__ }
+                map { $_->__meta__ } 
                 @$specified_class_name_arrayref
             ];        
         }
@@ -118,17 +132,12 @@ sub execute {
                 UR::Object::Type->get(data_source => $item->[0], table_name => $item->[1]);
             }
         }
-
+        
         for my $item (@$ds_table_list) {
             my ($data_source, $table_name) = @$item;
-            $self->_update_database_metadata_objects_for_schema_changes(
-                data_source => $data_source,
-                force_check_all_tables => $force_check_all_tables,
-                table_name => $table_name,
-            );
             for my $dd_class (qw/UR::DataSource::RDBMS::Table UR::DataSource::RDBMS::FkConstraint UR::DataSource::RDBMS::TableColumn/) {
                 push @data_dictionary_objects,
-                    $dd_class->get(data_source_obj => $data_source, table_name => $table_name);
+                    $dd_class->get(data_source => $data_source, table_name => $table_name);
             }
         }
     }
@@ -181,7 +190,7 @@ sub execute {
         # Get updates to it first.
         #
         
-        ##$DB::single = 1;
+        #$DB::single=1;
         
         for my $data_source (@target_data_sources) {
             # ensure the class has been lazy-loaded until UNIVERSAL::can is smarter...
@@ -196,22 +205,23 @@ sub execute {
                 return;
             }
         }
-
+    
         #
         # Summarize the database changes by table.  We'll create/update/delete the class which goes with that table.
         #
-
-        ##$DB::single = 1;
-
+    
+        #$DB::single = 1;
+   
         my $cx = UR::Context->current; 
         for my $dd_class (qw/UR::DataSource::RDBMS::Table UR::DataSource::RDBMS::FkConstraint UR::DataSource::RDBMS::TableColumn/) {
             push @data_dictionary_objects, 
-                grep { $force_rewrite_all_classes ? 1 : $_->__changes__ or exists($_->{'db_saved_uncommitted'}) } 
+                grep { $force_rewrite_all_classes ? 1 : $_->__changes__ } 
                 $cx->all_objects_loaded($dd_class);
-
+    
             my $ghost_class = $dd_class . "::Ghost";
             push @data_dictionary_objects, $cx->all_objects_loaded($ghost_class);
         }
+        
     }
     
     # The @data_dictionary_objects array has all dd meta which should be used to rewrite classes.
@@ -251,7 +261,7 @@ sub execute {
     # Update the classes based-on changes to the database schemas
     #
 
-    ##$DB::single = 1;
+    #$DB::single = 1;
 
     if (@data_dictionary_objects) {
         $self->status_message("Found " . keys(%changed_tables) . " tables with changes.") unless $force_rewrite_all_classes;
@@ -291,7 +301,7 @@ sub execute {
     $self->status_message("Saving metadata changes...");
     my $sync_success = UR::Context->_sync_databases();
     unless ($sync_success) {
-        ##$DB::single = 1;
+        #$DB::single=1;
         $self->error_message("Metadata sync_database failed");
         UR::Context->_rollback_databases();
         return;
@@ -304,7 +314,7 @@ sub execute {
     # Right now, it's done with a _load() override, no data_source, and this block of code. :(
     #
 
-    ##$DB::single = 1;
+    #$DB::single = 1;
 
     my $cx = UR::Context->current;
     my @changed_class_meta_objects;
@@ -361,7 +371,7 @@ sub execute {
     $self->status_message("Committing changes to data sources...");
 
     unless (UR::Context->_commit_databases()) {
-        ##$DB::single = 1;
+        #$DB::single=1;
         $self->error_message("Metadata commit failed");
         return;
     }
@@ -412,7 +422,6 @@ sub _update_database_metadata_objects_for_schema_changes {
     my ($self, %params) = @_;
     my $data_source = delete $params{data_source};
     my $force_check_all_tables = delete $params{force_check_all_tables};
-    my $table_name = delete $params{table_name};
     die "unknown params " . Dumper(\%params) if keys %params;
 
     #$data_source = $data_source->class;
@@ -431,13 +440,11 @@ sub _update_database_metadata_objects_for_schema_changes {
 
     # from the database now
     my @current_table_names = $data_source->_get_table_names_from_data_dictionary();
-    my %current_table_names = map { s/"|'//g; $_ => $_ } @current_table_names;
+    my %current_table_names = map { s/"|'//g; uc($_) => $_ } @current_table_names;
 
-    my %all_table_names = $table_name
-                            ? ( $table_name => 1 )
-                            : ( %current_table_names, %previous_table_names);
+    my %all_table_names = (%current_table_names, %previous_table_names);
 
-    my $new_object_revision = $UR::Context::current->now();
+    my $new_object_revision = UR::Time->now();
 
     # handle tables which are new/updated by updating the class
     my (@create,@delete,@update);
@@ -450,6 +457,8 @@ sub _update_database_metadata_objects_for_schema_changes {
         my $last_recorded_ddl_time;
         my $last_object_revision;
 
+        # UR always keeps table names stored in upper-case.  Some databases (mysql)
+        # are case sensitive when querying the data dictionary
         my $db_table_name = $current_table_names{$table_name};
 
         eval {
@@ -486,7 +495,7 @@ sub _update_database_metadata_objects_for_schema_changes {
                 my $this_update = $last_ddl_time_for_table_name->{$table_name} || "<unknown date>";
                 my $table_object = $data_source->refresh_database_metadata_for_table_name($db_table_name);
                 unless ($table_object) {
-                    ##$DB::single = 1;
+                    #$DB::single = 1;
                     print;
                 }
                 my @changes =
@@ -583,6 +592,50 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
     my $namespace = $self->namespace_name;
 
+=cut
+
+    $self->status_message("Using filesystem classes for namespace \"$namespace\" (this may be slow)");
+    my @material_classes = $namespace->get_material_classes;
+
+
+    $self->status_message("Verifying class/table relationships...");
+    my %table_ids_used;
+    for my $class (sort { $a->class_name cmp $b->class_name } @material_classes) {
+        my $table_name  = $class->table_name;
+        next unless $table_name;
+
+        my $class_name  = $class->class_name;
+
+        if (my $prev_class_name = $table_ids_used{$table_name}) {
+            $self->error_message(
+                sprintf(
+                    "C %-40s uses table %-32s, but so does %-40s" . "\n",
+                    $class_name, $table_name, $prev_class_name
+                )
+            );
+            return;
+        }
+
+        my $data_source = $class->data_source;
+
+        my $table = UR::DataSource::RDBMS::Table->get(data_source => $data_source, table_name => $table_name)
+                    ||
+                    UR::DataSource::RDBMS::Table::Ghost->get(data_source => $data_source, table_name => $table_name);
+
+        unless ($table) {
+            $self->error_message(
+                sprintf(
+                    "C %-32s %-32s is referenced by class %-40s but cannot be found!?" . "\n",
+                    $data_source, $table_name, $class_name
+                )
+            );
+            return;
+        }
+        $table_ids_used{$table_name} = $class;
+    }
+
+=cut
+
     $self->status_message("Updating classes...");
 
     my %dd_changes_by_class = (
@@ -598,8 +651,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $bucket = $dd_changes_by_class{$changed_class};
         push @$bucket, $changed_obj;
     }
-    my $sorter = sub($$) { no warnings 'uninitialized';
-                        $_[0]->table_name cmp $_[1]->table_name || $_[0]->id cmp $_[1]->id
+    my $sorter = sub { no warnings 'uninitialized';
+                        $a->table_name cmp $b->table_name || $a->id cmp $b->id
                      };
 
     # FKs are special, in that they might change names, but we use the name as the "id".
@@ -623,11 +676,20 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         }
 
         my $table = $fk->get_table;
-        my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
-                                                          table_name  => $table->table_name);
+        # FIXME should this use $data_source->get_class_meta_for_table($table) instead?
+        my $class = 
+            UR::Object::Type->get(
+                data_source_id => $table->data_source,
+                table_name     => $table->table_name,
+            )
+            ||
+            UR::Object::Type::Ghost->get(
+                data_source_id => $table->data_source,
+                table_name     => $table->table_name,
+            );
 
         unless ($class) {
-            ##$DB::single = 1;
+            #$DB::single = 1;
             $self->status_message(sprintf("~ No class found for deleted foreign key constraint %-32s %-32s\n",$table->table_name, $fk->id));
             next;
         }
@@ -660,8 +722,11 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         }
         my $column_name = $column->column_name;
 
-        my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
-                                                          table_name  => $table->table_name);
+        # FIXME should this use $data_source->get_class_meta_for_table($table) instead?
+        my $class = UR::Object::Type->get(
+            data_source_id => $table->data_source,
+            table_name     => $table->table_name,
+        );
         unless ($class) {
             $self->status_message(sprintf("~ No class found for deleted column %-32s %-32s\n", $table->table_name, $column_name));
             next;
@@ -706,7 +771,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
         if (not defined UR::Context->_get_committed_property_value($table,'table_name')) {
             print Data::Dumper::Dumper($table);
-            ##$DB::single = 1;
+            #$DB::single = 1;
         }
         # FIXME should this use $data_source->get_class_meta_for_table($table) instead?
         my $committed_data_source_id = UR::Context->_get_committed_property_value($table,'data_source');
@@ -754,14 +819,14 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     # but rolls back transactions between those calls.
     $self->{'_class_meta_cache'} = {};
 
-    ##$DB::single = 1;
+    #$DB::single = 1;
 
     #
     # EXISTING DD OBJECTS
     #
     # TABLE
     for my $table (sort $sorter @{ $dd_changes_by_class{"UR::DataSource::RDBMS::Table"} }) {
-        my $table_name = $table->table_name;
+        my $table_name = uc $table->table_name;
         my $data_source = $table->data_source;
 
         my $class = $self->_get_class_meta_for_table_name(data_source => $data_source,
@@ -796,7 +861,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         else {
             # create
             my $data_source = $table->data_source;
-            my $data_source_id = (ref $data_source ? $data_source->id : $data_source);
             my $class_name = $data_source->resolve_class_name_for_table_name($table_name,$table->table_type);
             unless ($class_name) {
                 Carp::confess(
@@ -809,13 +873,12 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             # new one actually has a table, then this is just another schema change and
             # not an error.  Set the table_name attribute and go on...
             my $class = UR::Object::Type->get(class_name => $class_name);
-            my $prev_table_name = ($class ? $class->table_name : undef);
-            my $prev_data_source_id = ($class ? $class->data_source_id : undef);
+            my $prev_table_name = $class->table_name if ($class);
             if ($class && $prev_table_name) {
 
                 Carp::confess(
-                    "Class $class_name already exists for table '$prev_table_name' in $prev_data_source_id."
-                    . "  Cannot generate class for $table_name in $data_source_id."
+                    "Class $class_name already exists for table '$prev_table_name'."
+                    . "  Cannot generate class for $table_name."
                 );
             }
 
@@ -827,14 +890,26 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                              $table_name)
                      );
 
+            my $type_name = $data_source->resolve_type_name_for_table_name($table_name);
+            $type_name .= ' view' if ($table->table_type =~ m/view/i);
+            unless ($type_name) {
+                Carp::confess(
+                    "Failed to resolve a type name for new table "
+                    . $table_name
+                );
+            }
+
             if ($class) {
+                $class->type_name($type_name);
                 $class->doc($table->remarks ? $table->remarks: undef);
                 $class->data_source($data_source);
                 $class->table_name($table_name);
+                # FIXME we should pick one of these names to standarize on
                 $class->er_role($table->er_type);
             } else {
                 $class = UR::Object::Type->create(
                             class_name => $class_name,
+                            type_name => $type_name,
                             doc => ($table->remarks ? $table->remarks: undef),
                             data_source_id => $data_source,
                             table_name => $table_name,
@@ -849,6 +924,15 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                     );
                 }
             }
+
+            #unless ($class->class_name->isa('UR::Entity')) {
+            #    my $inheritance = UR::Object::Inheritance->create(
+            #        class_name => $class->class_name,
+            #        parent_class_name => "UR::Entity",
+            #        inheritance_priority => 0,
+            #    );
+            #    Carp::confess("Failed to generate inheritance link!?") unless $inheritance;
+            #}
         }
     } # next table
 
@@ -857,48 +941,44 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     $self->status_message($_) foreach @saved_removed_column_messages;
 
     # COLUMN
+    my @column_property_translations = (
+        ['data_type'    => 'data_type'],
+        ['data_length'  => 'data_length'],
+        ['nullable'     => 'is_optional', sub { (defined($_[0]) and ($_[0] eq "Y")) ? 1 : 0 } ],
+        ['remarks'      => 'doc'],
+    );
     
     for my $column (sort $sorter @{ $dd_changes_by_class{'UR::DataSource::RDBMS::TableColumn'} }) {
         my $table = $column->get_table;
         my $column_name = $column->column_name;
         my $data_source = $table->data_source;
-        my($ur_data_type, $default_length) = @{ $data_source->ur_data_type_for_data_source_data_type($column->data_type) };
-        my $ur_data_length = defined($column->data_length) ? $column->data_length : $default_length;
 
+        #my $class = UR::Object::Type->get(
+        #    data_source => $table->data_source,
+        #    table_name => $table->table_name,
+        #);
+        #my $class = $data_source->get_class_meta_for_table($table);
         my $class = $self->_get_class_meta_for_table_name(data_source => $data_source,
                                                           table_name => $table->table_name);
 
         unless ($class) {
+            #$DB::single = 1;
             $class = $self->_get_class_meta_for_table_name(data_source => $data_source,
                                                           table_name => $table->table_name);
             Carp::confess("Class object missing for table " . $table->table_name) unless $class;
         }
         my $class_name = $class->class_name;
         my $property;
+        $column_name = uc($column_name);
         foreach my $prop_object ( $class->direct_property_metas ) {
-            if (defined $prop_object->column_name and lc($prop_object->column_name) eq lc($column_name)) {
+            if (uc($prop_object->column_name) eq $column_name) {
                 $property = $prop_object;
                 last;
             }
-        }
+       }
 
         # We care less whether the column is new/updated, than whether there is property metadata for it.
         if ($property) {
-            my @column_property_translations = (
-                # [ column_name, property_name, conversion_sub(column_obj, value) ]
-                ['data_length'  => 'data_length',
-                    # lengths for these data types are based on the number of bytes used internally in the
-                    # database.  The UR-based objects will store the text version, which will always be longer,
-                    # making $obj->__errors__() complain about the length being out of bounds
-                    sub { my ($c, $av) = @_;  defined($av) ? $av : ($c->is_time_data ? undef : $ur_data_length) } ],
-                ['data_type'    => 'data_type',
-                    sub { my ($c, $av) = @_;  defined($ur_data_type) ? $ur_data_type : $av } ],
-                ['nullable'     => 'is_optional',
-                    sub { my ($c, $av) = @_; (defined($av) and ($av eq "Y")) ? 1 : 0 } ],
-                ['remarks'      => 'doc',
-                    # Ideally this would only use DB value ($av) if the last_ddl_time was newer.
-                    sub { my ($c, $av) = @_; defined($av) ? $av : $property->doc } ],
-            );
             # update
             for my $translation (@column_property_translations) {
                 my ($column_attr, $property_attr, $conversion_sub) = @$translation;
@@ -907,14 +987,22 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 no warnings;
                 if (UR::Context->_get_committed_property_value($column,$column_attr) ne $column->$column_attr) {
                     if ($conversion_sub) {
-                        $property->$property_attr($conversion_sub->($column, $column->$column_attr));
+                        $property->$property_attr($conversion_sub->($column->$column_attr));
                     }
                     else {
                         $property->$property_attr($column->$column_attr);
                     }
                 }
             }
+            $property->data_type($column->data_type);
+            # lengths for these data types are based on the number of bytes used internally in the
+            # database.  The UR-based objects will store the text version, which will always be longer,
+            # making $obj->__errors__() complain about the length being out of bounds
+            $property->data_length($column->is_time_data ? undef : $column->data_length);
 
+            $property->is_optional($column->nullable eq "Y" ? 1 : 0);
+            $property->doc($column->remarks);
+            
             if ($property->__changes__) {
                 no warnings;
                 $self->status_message(
@@ -938,35 +1026,35 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 );
             }
 
-            my $create_exception;
-            for (my $attempt = 0; $attempt < 3; $attempt++) {
-                $property_name = '_' . $property_name if $attempt;
-
-                $create_exception = do {
-                    local $@;
-                    eval {
-                        $property = UR::Object::Property->create(
-                            class_name     => $class_name,
-                            property_name  => $property_name,
-                            column_name    => $column_name,
-                            data_type      => $ur_data_type,
-                            data_length    => $ur_data_length,
-                            is_optional    => $column->nullable eq "Y" ? 1 : 0,
-                            is_volatile    => 0,
-                            doc            => $column->remarks,
-                            is_specified_in_module_header => 1,
-                        );
-                    };
-                    $@;
-                };
-                last if $property;
+            my $attribute_name = $data_source->resolve_attribute_name_for_column_name($column->column_name);
+            unless ($attribute_name) {
+                Carp::confess(
+                    "Failed to resolve a attribute name for new column "
+                    . $column->column_name
+                );
             }
 
-            no warnings 'uninitialized';
+            my $type_name = $class->type_name;
+
+            $property = UR::Object::Property->create(
+                class_name     => $class_name,
+                type_name      => $type_name,
+                attribute_name => $attribute_name,
+                property_name  => $property_name,
+                column_name    => $column_name,
+                data_type      => $column->data_type,
+                data_length    => $column->is_time_data ? undef : $column->data_length,
+                is_optional    => $column->nullable eq "Y" ? 1 : 0,
+                is_volatile    => 0,
+                doc            => $column->remarks,
+                is_specified_in_module_header => 1, 
+            );
+
+            no warnings;
             $self->status_message(
                 sprintf("A %-40s property %-16s for column %s.%s (%s %s)\n",
                                                         $class_name,
-                                                        $property->property_name,
+                                                        $property_name,
                                                         $table->table_name, 
                                                         $column_name,
                                                         $column->data_type,
@@ -974,16 +1062,21 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             );
             
             unless ($property) {
-                if ($create_exception =~ m/An object of class UR::Object::Property already exists/) {
-                    $self->warning_message("Conflicting property names already exist in class $class_name for column $column_name in table ".$table->table_name);
-                } else {
-                    Carp::confess(
-                            "Failed to create property $property_name on class $class_name. "
-                            . UR::Object::Property->error_message
-                    );
-                }
+                Carp::confess(
+                        "Failed to create property $property_name on class $class_name. "
+                        . UR::Object::Property->error_message
+                );
             }
         }
+        
+        # FIXME Moved the creating/setting of these properties up a bit, since the
+        # create() invovles a call into the class object containing the property, which handles
+        # updating the class' flat-format data.  Changing the 
+        #$property->data_type($column->data_type);
+        #$property->data_length($column->data_length);
+        #$property->is_optional($column->nullable eq "Y" ? 1 : 0);
+        #$property->doc($column->remarks);
+
     } # next column
 
     $self->status_message("Updating class ID properties...\n");
@@ -997,17 +1090,17 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
                                                           table_name => $table_name);
         my $class_name = $class->class_name;
+        my $type_name = $class->type_name;
         my @properties = UR::Object::Property->get(class_name => $class_name);
 
         unless (@properties) {
             $self->warning_message("no properties on class $class_name?");
-            ##$DB::single = 1;
+            #$DB::single = 1;
         }
 
         my @expected_pk_cols = grep { defined }
                                map { $_->column_name }
-                               grep { defined $_->is_id }
-                               @properties;
+                               $class->direct_id_property_metas;
         
         my @pk_cols = $table->primary_key_constraint_column_names;
         
@@ -1025,7 +1118,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my %pk_cols;
         for my $pos (1 .. @pk_cols) {
             my $pk_col = $pk_cols[$pos-1];
-            my ($property) = grep { defined($_->column_name) and ($_->column_name eq $pk_col) } @properties;
+            my ($property) = grep { defined($_->column_name) and (uc($_->column_name) eq uc($pk_col)) } @properties;
             
             unless ($property) {
                 # the column has been removed
@@ -1060,7 +1153,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         if ($property_meta && $property_meta->column_name && scalar($class_meta->direct_id_property_metas) > 1) {
             $self->warning_message("Class $class_name cannot have multiple ID properties when one concrete ID property is named 'id'. It will likely not function correctly unless it is renamed");
         }
-        unless (defined $property_meta->is_id) {
+        unless ($property_meta->is_id) {
             $self->warning_message("Class $class_name has a property named 'id' that is not an ID property.  It will likely not function correctly unless it is renamed");
         }
     }
@@ -1069,26 +1162,29 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
     $self->status_message("Updating class unique constraints...\n");
 
+    #$DB::single = 1;
+
     # UNIQUE CONSTRAINT / UNIQUE INDEX -> UNIQUE GROUP (loop table objecs since we have no PK DD objects)
     for my $table (sort $sorter @{ $dd_changes_by_class{'UR::DataSource::RDBMS::Table'} }) {
         # created/updated/unchanged
         # delete and re-create
 
+        #my $class = UR::Object::Type->get(
+        #    data_source => $table->data_source,
+        #    table_name => $table->table_name,
+        #);
+        #my $class = $table->__meta__();
         my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
                                                           table_name => $table->table_name);
         my $class_name = $class->class_name;
+        my $type_name = $class->type_name;
 
         my @properties = UR::Object::Property->get(class_name => $class_name);
 
         my @uc_names = $table->unique_constraint_names;
         for my $uc_name (@uc_names)
         {
-            eval { $class->remove_unique_constraint($uc_name) };
-            if ($@ =~ m/There is no constraint named/) {
-                next;  # it's OK if there's no UR metadata for this constraint yet
-            } elsif ($@) {
-                die $@;
-            }
+            $class->remove_unique_constraint($uc_name);
 
             my @uc_cols = map { ref($_) ? @$_ : $_ } $table->unique_constraint_column_names($uc_name);
             my @uc_property_names;
@@ -1097,7 +1193,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 my ($property) = grep { defined($_->column_name) and ($_->column_name eq $uc_col) } @properties;
                 unless ($property) {
                     $self->warning_message("No property found for column $uc_col for unique constraint $uc_name");
-                    #$DB::single = 1;
+                    $DB::single=1;
                     next;
                 }
                 push @uc_property_names, $property->property_name;
@@ -1144,6 +1240,10 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class_name = $class->class_name;
         my $r_class_name = $r_class->class_name;
 
+        my $type_name = $class->type_name;
+        my $r_type_name = $r_class->type_name;
+
+
         # Create an object-accessor property to go with this FK
         # First we have to figure out a proper delegation name
         # which is a rather convoluted process
@@ -1171,7 +1271,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                             );
             unless ($r_property) {
                 Carp::cluck("Failed to find a property for column $r_column_name on class $r_class_name");
-                #$DB::single = 1;
+                $DB::single = 1;
                 next FK;
             }
             push @r_properties,$r_property;
@@ -1189,10 +1289,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             }
         }
 
-        my @r_class_name_parts = split('::', $r_class->class_name);
-        shift @r_class_name_parts;  # drop the namespace name
-        my $delegation_name = lc(join('_', @r_class_name_parts));
-
+        my $delegation_name = $r_class->type_name;
+        $delegation_name =~ s/ /_/g;
         if ($matched) {
             $delegation_name = $delegation_name . "_" . $prefix if $prefix;
             $delegation_name .= ($suffix !~ /\D/ ? "" : "_") . $suffix if $suffix;
@@ -1214,10 +1312,12 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         {
             my $column_name = $column_names[$i];
             my $property = $properties[$i];
+            my $attribute_name = $property->attribute_name;
             my $property_name = $property_names[$i];
 
             my $r_column_name = $r_column_names[$i];
             my $r_property = $r_properties[$i];
+            my $r_attribute_name = $r_property->attribute_name;
             my $r_property_name = $r_property_names[$i];
         }
 
