@@ -283,6 +283,9 @@ sub mk_indirect_rw_accessor {
             # which are not id properties.
         
             my $my_property_meta = $class_name->__meta__->property_meta_for_name($accessor_name);
+            unless ($my_property_meta) {
+                Carp::confess("Failed to find property meta for $accessor_name on $class_name!");
+            }
             $is_many = $my_property_meta->is_many;
 
             $via_property_meta ||= $class_name->__meta__->property_meta_for_name($via);
@@ -761,11 +764,10 @@ sub mk_object_set_accessors {
         if ($r_class_meta and not $reverse_as) {
             # we have a real class on the other end, and it did not specify how to link back to us
             # try to infer how, otherwise fall back to the same logic we use with "primitives"
-            my @possible_relationships = UR::Object::Reference->get(
-                class_name => $r_class_name,
-                r_class_name => $class_name
-            );
-            #print "got " . join(", ", map { $_->id } @possible_relationships) . "\n";
+            my @possible_relationships = grep { $_->data_type eq $class_name }
+                                         grep { defined $_->data_type }
+                                         $r_class_meta->all_property_metas();
+
             if (@possible_relationships > 1) {
                 Carp::croak "$class_name has an ambiguous definition for property \"$singular_name\"."
                     . "  The target class $r_class_name has " . scalar(@possible_relationships) 
@@ -774,7 +776,7 @@ sub mk_object_set_accessors {
                     . join(",",map { '"' . $_->delegation_name . '"' } @possible_relationships) . ".\n";
             }
             elsif (@possible_relationships == 1) {
-                $reverse_as = $possible_relationships[0]->delegation_name;
+                $reverse_as = $possible_relationships[0]->property_name;
             }
             elsif (@possible_relationships == 0) {
                 # we now fall through to the logic below and try direct arrayref storage
@@ -800,20 +802,15 @@ sub mk_object_set_accessors {
             unless ($property_meta) {
                 Carp::croak "Can't resolve reverse relationship $class_name -> $plural_name.  Remote class $r_class_name has no property $reverse_as";
             }
-            my @property_links = $property_meta->id_by_property_links;
-            #my @property_links = UR::Object::Reference::Property->get(tha_id => $r_class_name . '::' . $reverse_as); 
-            unless (@property_links) {
-                #$DB::single = 1;
-                Carp::croak("No property links for $r_class_name -> $reverse_as?  Cannot build accessor for $singular_name/$plural_name relationship.");
-            }
+            my @property_links = $property_meta->get_property_name_pairs_for_join;
             my @get_params;
             for my $link (@property_links) {
-                my $my_property_name = $link->r_property_name;
+                my $my_property_name = $link->[1];
                 push @property_names, $my_property_name;
                 unless ($obj->can($my_property_name)) {
                     Carp::croak "Cannot handle indirect relationship $r_class_name -> $reverse_as.  Class $class_name has no property named $my_property_name";
                 }
-                push @get_params, $link->property_name, ($obj->$my_property_name || undef);
+                push @get_params, $link->[0], ($obj->$my_property_name || undef);
             }
             if (my $id_class_by = $property_meta->id_class_by) {
                 push @get_params, $id_class_by, $class_name;
