@@ -3,8 +3,6 @@ package UR::Object::Type::AccessorWriter::Sum;
 
 use strict;
 use warnings;
-require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 sub calculate {
     my $self = shift;
