@@ -2,16 +2,10 @@ package UR::Object::View::Default::Gtk;
 
 use strict;
 use warnings;
-require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => 'UR::Object::View',
-    has_constant => [
-        perspective => { value => 'default' },
-        toolkit => { value => 'gtk' },
-    ]
 );
 
 sub _create_widget {
@@ -36,11 +30,7 @@ sub _update_view_from_subject {
         $text .= " (REUSED ADDR)\n";
     } else {
         $text .= "\n";
-        my @sorted_aspects = map { $_->[1] }
-                             sort { $a->[0] <=> $b->[0] }
-                             map { [ $_->position, $_ ] }
-                             @aspects;
-        for my $aspect (@sorted_aspects) {
+        for my $aspect (sort { $a->position <=> $b->position } @aspects) {       
             my $label = $aspect->label;
             $text .= "\n" . $label . ": ";
             if ($subject) {
